;;; Gash --- Guile As SHell
;;; Copyright © 2018,2019 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
;;;
;;; This file is part of Gash.
;;;
;;; Gash is free software; you can redistribute it and/or modify it
;;; under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or (at
;;; your option) any later version.
;;;
;;; Gash is distributed in the hope that it will be useful, but WITHOUT ANY
;;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;;; details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with Gash.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:

;;; Code:

(define-module (gash core-utils gzip)
  #:use-module (ice-9 getopt-long)
  #:use-module (srfi srfi-1)

  #:use-module (gash core-utils config)
  #:use-module (gash gzip)
  #:use-module (gash guix-utils)
  #:export (gzip))

(define (gzip . args)
  (let* ((option-spec
	  '((bits (single-char #\b) (value #t))
            (decompress (single-char #\d))
            (force (single-char #\f))
            (help (single-char #\h))
            (stdout (single-char #\c))
	    (verbose (single-char #\v))
            (version (single-char #\V))))
	 (options (getopt-long args option-spec))
         (decompress? (option-ref options 'decompress #f))
         (stdout? (option-ref options 'stdout #f))
	 (files (option-ref options '() '()))
	 (help? (option-ref options 'help #f))
	 (usage? (and (not help?) (or (and (null? files) (isatty? (current-input-port))))))
	 (verbose? (option-ref options 'verbose #f))
         (version? (option-ref options 'version #f)))
    (cond (version? (format #t "gzip (GASH) ~a\n" %version) (exit 0))
          ((or help? usage?) (format (if usage? (current-error-port) #t)
                                     "\
Usage: gzip [OPTION]... [FILE]...
  -c, --stdout      write on standard output, keep original files unchanged
  -d, --decompress  decompress
  -f, --force       force overwrite of output file and compress links
  -h, --help        display this help
  -v, --verbose     show compression ratio
  -V, --version     display version
")
           (exit (if usage? 2 0)))
          (decompress? (if (and (pair? files) (not stdout?)) (gunzip-file (car files) verbose?)
                           (gunzip-port
                            (if (null? files) (current-input-port)
                                (open-input-file (car files))) (current-output-port) verbose?)))
          (else (format (current-error-port) "gzip: compression not supported\n")
                (exit 1)))))

(define main gzip)
